// $Id:$   -*-c++-*-
// by Chris Crawford <chris2@lns.mit.edu> 2006/09/27
// 
// root -l mcstas.root chop_width.C(15,1,1e6)

#include "tx_pol_cuts.C"

float chop_phi[]={-1, 80, 107, 128, 149};
float chop_ang[]={-1, 125, 168, 201, 234};
float width[2];


// chopper roll-off
// c=chop_tx("f1",125,80)

float* fit_edge(TCut c, int nev=5e6, bool draw=true)
{
  TH1 *ha,*hc,*hn,*hd;
  int nbin=100;

  if (gROOT->FindObject("ha")) gROOT->FindObject("ha")->Delete();
  if (gROOT->FindObject("hc")) gROOT->FindObject("hc")->Delete();
  if (gROOT->FindObject("hn")) gROOT->FindObject("hn")->Delete();
  if (gROOT->FindObject("hd")) gROOT->FindObject("hd")->Delete();

  delete ha; ha=new TH1F("ha","ha",nbin,0,10);
  delete hc; hc=new TH1F("hc","hc",nbin,0,10);
  delete hn; hn=new TH1F("hn","hn",nbin,0,10);
  delete hd; hd=new TH1F("hd","hd",nbin,0,10);

  ha->SetLineColor(kBlack);
  hc->SetLineColor(kRed);
  hn->SetLineColor(kGreen);
  hd->SetLineColor(kBlue);

  mcstas->Draw("l>>ha","p5",draw?"":"goff",nev);
  mcstas->Draw("l>>hc","p5"*c,draw?"same":"goff",nev);
  hn->Divide(hc,ha);
  hn->Scale(nev);
  if (draw) hn->Draw("same");
  for (int i=0;i<nbin;++i)
    hd->SetBinContent(i, fabs(hn->GetBinContent(i+1) -
			      hn->GetBinContent(i)));
  hd->Scale(10);
  if (draw) hd->Draw("same");
  float range[2][2]={{0,4.5},{4.5,9}};
  TF1* g[2];
  for (int i=0;i<2;i++) {
    TString gname=Form("gr%d",i);
    g[i]=new TF1(gname,"gaus");
    g[i]->SetLineWidth(1);
    g[i]->SetLineColor(kBlue);
    hd->Fit(g[i],draw?"lq+":"lq0+","same",range[i][0],range[i][1]);
    width[i]=hd->GetFunction(gname)->GetParameter(2);
  }
  return width;
}


// calculate widths as a function of chopper phases
// display graph of widths
void width_fn(int n1, int n2=0, float max=15, float step=1, int nev=5e6,
	      bool newcanvas=true, int legend=true )
{
  TString cname=Form("chopper %d", n1);
  if (n2) cname+=Form(",%d", n2);
  cout<<endl<<"===== "<<cname<<" ====="<<endl<<endl;
  if (newcanvas) TCanvas* c=new TCanvas(cname,cname);
  TGraph *ga[4], *gb[4];
  TLegend *la=new TLegend(.5,.1,.7,.4);
  TLegend *lb=new TLegend(.7,.1,.9,.4);
  la->SetBorderSize(0);		lb->SetBorderSize(0);
  la->SetFillStyle(0);		lb->SetFillStyle(0);
  la->SetHeader("leading edge");lb->SetHeader("trailing edge");

  for (int j=0;j<(!n2?2:4);++j) {
    char* fg_ord[4]={"ff","gg","fg","gf"};
    TString var1=Form("%c%d",fg_ord[j][0],n1);
    TString var2=Form("%c%d",fg_ord[j][1],n2);
    TString var=!n2? var1 : var1+" * "+var2;
    ga[j]=new TGraph; gb[j]=new TGraph;
    la->AddEntry(ga[j],var,"l");
    lb->AddEntry(gb[j],var,"l");
    cout<<"     "<<var<<endl;

    TCut c1,c2;
    for (float i=-max;i<=max;i+=step) {
      c1=chop_tx(var1,chop_phi[n1]-i,chop_ang[n1]);
      c2=chop_tx(var2,chop_phi[n2]+i,chop_ang[n2]);
      fit_edge(!n2? c1: c1*c2, nev, 0);
      printf("%3.0f:   %8f %8f",i,width[0],width[1]);
      ga[j]->SetPoint(ga[j]->GetN(),i,width[0]);

      c1=chop_tx(var1,chop_phi[n1],chop_ang[n1]-i);
      c2=chop_tx(var2,chop_phi[n2],chop_ang[n2]+i);
      fit_edge(!n2? c1: c1*c2, nev, 0);
      printf("   %8f %8f\n",width[0],width[1]);
      gb[j]->SetPoint(gb[j]->GetN(),i,width[1]);
    }

    ga[j]->GetXaxis()->SetTitle("#Delta phase [deg]");
    ga[j]->GetYaxis()->SetTitle("rolloff width [#AA]");
    ga[j]->GetYaxis()->SetTitleOffset(2);
    ga[j]->SetMaximum(0.3);
    ga[j]->SetMinimum(0.1);
    float gcol[4]={kBlack,kBlue, kRed,kMagenta};
    ga[j]->SetLineColor(gcol[j]);
    ga[j]->SetMarkerColor(gcol[j]);
    ga[j]->SetMarkerStyle(6);
    ga[j]->Draw(j?"pc":"apc");
    gb[j]->SetLineColor(gcol[j]);
    gb[j]->SetMarkerColor(gcol[j]);
    gb[j]->SetMarkerStyle(6);
    gb[j]->SetLineStyle(2);
    gb[j]->Draw("pc");
  }

  if (legend) la->Draw(), lb->Draw();
  gPad->Update();
}

// plot canvas of width plots
// for various chopper combinations

void chop_width(float max=15, float step=1, int nev=5e6)
{
  gROOT->SetStyle("Plain");
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.05);
  TCanvas* c=new TCanvas("rw","rolloff widths");
  c->Divide(3,2,.001,.001);

  c->cd(1); width_fn(1,2,max,step,nev,0,0); TLatex* l=new TLatex(-15,.305,"Choppers 1 & 2"); l->Draw(); gPad->Update();
  c->cd(2); width_fn(2,3,max,step,nev,0,0); TLatex* l=new TLatex(-15,.305,"Choppers 2 & 3"); l->Draw(); gPad->Update();
  c->cd(3); width_fn(3,4,max,step,nev,0,0); TLatex* l=new TLatex(-15,.305,"Choppers 3 & 4"); l->Draw(); gPad->Update();
  c->cd(4); width_fn(1,3,max,step,nev,0,0); TLatex* l=new TLatex(-15,.305,"Choppers 1 & 3"); l->Draw(); gPad->Update();
  c->cd(5); width_fn(2,4,max,step,nev,0,0); TLatex* l=new TLatex(-15,.305,"Choppers 2 & 4"); l->Draw(); gPad->Update();
  c->cd(6); width_fn(1,4,max,step,nev,0,0); TLatex* l=new TLatex(-15,.305,"Choppers 1 & 4"); l->Draw(); gPad->Update();

  c->cd(3);
  TLegend *la=new TLegend(.25,.50,.65,.90);
  TLegend *lb=new TLegend(.60,.50,.99,.90);
  la->SetBorderSize(0);		lb->SetBorderSize(0);
  la->SetFillStyle(0);		lb->SetFillStyle(0);
  la->SetHeader("leading edge");lb->SetHeader("trailing edge");
  char* ldesc[4]={"cw,cw","ccw,ccw","cw,ccw","ccw,cw"};
  for (int j=0;j<4;++j) {
    la->AddEntry((TGraph*)gPad->GetListOfPrimitives()->At(1+2*j),ldesc[j],"l");
    lb->AddEntry((TGraph*)gPad->GetListOfPrimitives()->At(2+2*j),ldesc[j],"l");
  }
  la->Draw(); lb->Draw(); gPad->Update();

}

#ifndef __CINT__
int main(int argc, char** argv)
{
  gApplication = new TApplication("chop_width", &argc, argv);
  chop_width();
  //gApplication->Run();
}
#endif
