// $Id:$   -*-c++-*-
// by Chris Crawford <chris2@lns.mit.edu> 2006/09/08
// ntuple.cc an interface to ROOT ntuples
// g++-3.3 -c `root-config --cflags` ntuple.cc

#include "ntuple.h"
#include "TString.h"
#include "TFile.h"
#include "TNtuple.h"

TFile*   _fp_=0;
TNtuple* _ntp_=0;

float* ntuple_pointer(float* fill, char* var)
{
  // return pointer to fill field described by var
  class TBranch* br=_ntp_->FindBranch(var);
  if (!br) printf("ERROR: undefined ntuple variable '%s'\n;", var);
  return fill + _ntp_->GetListOfBranches()->IndexOf(br);
}

void ntuple_clear(int cnt, float* fill, float clear)
{
  for (int i=0;i<cnt;++i) fill[i]=clear;
}

TString ntuple_expand(const char* fields)
{
  // parse and expand state suffixes ending in ';'
  TString fst;
  const char* pt=fields;
  char buf[strlen(fields)];
  static char* cst[]={"x","y","z","vx","vy","vz","t","p"};

  while (*pt) {
    char *pf=buf, ch;
    while (1) {
      if ((ch = *pt)) pt++;	// advance to next char
      if (ch==':' || !ch) {	// copy simple leaf
	*pf=0;
	fst=fst.Append(buf).Append(":");
	break;
      }
      else if (ch==';') {	// expand state suffix
	*pf=0;
	for (int i=0;i<8;++i)
	  fst.Append(cst[i]).Append(buf).Append(":");
	break;
      }
      else *pf++=ch;
    }
  }
  fst.Remove(TString::kBoth,':');
  return fst;
}

int ntuple_open(char* file, char* desc, char* fields)
{
  _fp_=new TFile(TString(file)+".root","recreate");
  _fp_->SetCompressionLevel(0);
  _ntp_=new TNtuple(file, desc, ntuple_expand(fields));
  if (_ntp_) return _ntp_->GetNbranches();
  else return 0;
}

void ntuple_fill(float* entries)
{
  if (_ntp_) _ntp_->Fill(entries);
}

void ntuple_close()
{
  if (_ntp_) _ntp_->Write();
  if (_fp_) _fp_->Close();
  delete _fp_;
  _fp_=0;
}
