// $Id:$   -*-c++-*-
// by Chris Crawford <chris2@lns.mit.edu> 2006/09/25
// 
// $ root -l tx_pol_cuts.C ntp5/mcstas.root

#include "TCut.h"


/// Chopper
//
// nominal phases

double cf[]={  73,  98, 117, 136 };
double cw[]={ 125, 168, 201, 234 };

void print_naive_angles()
{
  int ch_ind[6][2]={{1,2},{2,3},{3,4},{1,3},{2,4},{1,4}};
  for (int iv=0; iv<6;iv++) {
    cout<<"200"; cout<<"\t";
    cout<<int(cf[ch_ind[iv][0]-1]); cout<<"\t";
    cout<<int(cw[ch_ind[iv][0]-1]); cout<<"\t";
    cout<<int(cf[ch_ind[iv][1]-1]); cout<<"\t";
    cout<<int(cw[ch_ind[iv][1]-1]); cout<<"\t";
    cout<<endl;
  }
}

// var = a##i, eg. "f1" where
//   a="f"(cw),"g"(ccw) and i="1","2","3","4"
// 0<= var < 360 required range of chopper variable
// 0<= phi < 360 phase at opening
// 0<= open<=360 opening angle

TCut chop_tx(char* var, double phi, double open)
{
  while (phi<0) phi+=360;
  while (phi>=360) phi-=360;
  char* str = (phi+open<360) ?
    Form("%f<%s&&%s<%f", phi, var,var, phi+open) :
    Form("%f<%s||%s<%f", phi, var,var, phi+open-360);
  return TCut(str);
}

TCut chop_tx(char* var)
{
  if (!var||!var[0]||var[1]<'1'||var[1]>'4') return TCut("");
  return chop_tx(var,cf[var[1]-'1'],cw[var[1]-'1']);
}


/// He3 cell
//
// ksig = 0.07855/(Amg.cm.Ang)  =  5327 b * 2200 m/s * mn/h
// 0<pol<1, nl = 4.0 Amg.cm

double ksig=0.07855;

TCut he3_tx(double pol=0.6, double nl=4.0)
{
  return Form("exp(%f*l)*cosh(%f*l)", -nl*ksig, pol*nl*ksig);
}

TCut he3_pol(double pol=0.6, double nl=4.0)
{
  return Form("tanh(%f*l)", pol*nl*ksig);
}


/// RFSF
//
// dist [m]	effective distance of the RFSF (a tuning parameter)
// -360<=phi	(phase = t*freq*360deg) when the spin flipper starts
// freq=60Hz	accelerator pulse frequency
// 111		circumvents fmod(-x) quirk
// abs_np	number of pulses away from correct one
// k_eps	average efficiency over location in spin sequence
// eps		efficiency of spin flipper (when on; else -1)
// note:	final result also averaged over up/down pulses
//		we assume that polarization asymmetry cancelled by geometry

TCut rfsf_pol(double phi=200, double dist=15.868, double freq=60.)
{
  char* abs_np=Form("((abs(t_sf*%g-%g)+0.5)%%8)", freq, phi/360+0.5);	// 0,1,2,3,4
  char* k_eps =Form("((8728785<<3*%s&14680064)/2097152-2)/2", abs_np);	// 1, -1/2, 0, 1/2, -1
  char* eps   =Form("cos(%g/vz_sf/(fmod(t_sf+%g,%g)+%g))",		// RFSF efficiency (+/-)
		    acos(-1)*dist, (111-phi/360)/freq, 1/freq, phi/360/freq );
  return Form("%s*(1-%s)/2", k_eps, eps);				// RFSF on/off, spin seq ave
}


/// LH2 para-hydrogen target
//
// cutoff [meV]	is the highest neutron energy in which polarization is preserved
// kmn = 2/mn = 191312.97 (m/s)^2/meV;  v=sqrt(2E/mn)=sqrt(kmn*E)

double kmn=191312.97;

TCut lh2_pol(double cutoff=15)
{
  return Form("v<%f",sqrt(kmn*cutoff));
}


/// end of cuts

void tx_pol_cuts()
{
}

#ifndef __CINT__
int main(int argc, char** argv)
{
  gApplication = new TApplication("tx_pol_cuts", &argc, argv);
  tx_pol_cuts();
}
#endif
